using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Dynamic;
using System.Net.Mime;
using System.Net.Sockets;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

namespace Harjotustyo;

/// @author Niko Koskinen
/// @version 17.11.2023
/// 
/// <summary>
/// Vaatii korjausta:
/// Pelaajan kuva (ja samalla vihollisen kuva) -- Hoidossa
/// 
/// Vihollisen spawnaus seinän sisään
/// pelin päättyminen -- hoidossa jollain tasolla....
/// top 3 lista (mahd taulukolla ja loopilla?) 2 kpl aijoille ja kolikoille
/// 
/// Siistiminen
/// virheiden korjaus
/// 
/// </summary>

public class Vihollinen : PhysicsObject
{
    private IntMeter vihollisenTerveys = new IntMeter(2, 0, 2);
    
    public IntMeter TerveysLaskuri
    {
        get { return vihollisenTerveys; }
    }

    
    public Vihollinen(double leveys, double korkeus)
        : base(leveys, korkeus)
    {
        vihollisenTerveys.LowerLimit += delegate { this.Destroy(); };
    }

    
}

public class Auto : PhysicsObject
{
    private IntMeter PelaajanTerveys = new IntMeter(4, 0, 4);

    public IntMeter TerveysLaskuri
    {
        get { return PelaajanTerveys; }
    }

    public Auto(double leveys, double korkeus) : base(40, 20)
    {
        PelaajanTerveys.LowerLimit += delegate { this.Destroy(); };
    }
    
}

public class Harjotustyo : PhysicsGame
{
    
    private IntMeter pistelaskuri;                  //Luo pistelaskurin, joka laskee kolikoita (pisteitä)
    
    private DoubleMeter elamalaskuri;               //Luo pelaajalle ja vihollisille elämälaskurin
    
    private EasyHighScore topLista1 = new EasyHighScore();
    
    private EasyHighScore topLista2 = new EasyHighScore();
    
    Image pelaajanKuva = LoadImage("UusiAuto.png");  //Auton kuva pelaajaa varten

    Auto pelaaja;

    public override void Begin()
    {
        Alkuvalikko();

    }

    /// <summary>
    /// Luodaan pelaaja ja määritellään sille ominaisuuksia
    /// </summary>
    ///

    public void LuoPelaaja()
    {
        pelaaja = new Auto(40, 20);
                                                                //Pelaajan muoto tai kuva
        pelaaja.Shape = Shape.Rectangle;
        pelaaja.Tag = "pelaaja";
        Add(pelaaja); 
        pelaaja.MaxVelocity = 500;         // maks nopeus
        pelaaja.Restitution = 0.5;         
        pelaaja.MomentOfInertia = 200;
        pelaaja.MaxAngularVelocity = 50;
        AddCollisionHandler(pelaaja, "kolikko", KeraaKolikon);
        AddCollisionHandler(pelaaja, "vihu", Kolari);
        
        Camera.Follow(pelaaja);
        
        Keyboard.Listen(Key.Left, ButtonState.Down, KaannaPelaajaav, "Käänny vasemmalle", pelaaja);
        Keyboard.Listen(Key.Left, ButtonState.Released, Pysayta, null, pelaaja);
        Keyboard.Listen(Key.Right, ButtonState.Down, KaannaPelaajaao, "Käänny oikealle", pelaaja);
        Keyboard.Listen(Key.Right, ButtonState.Released, Pysayta, null, pelaaja);
        Keyboard.Listen(Key.Up, ButtonState.Down, LiikutaPelaajaaE, "Eteen", pelaaja);
        Keyboard.Listen(Key.Up, ButtonState.Released, Hidasta, null, pelaaja);
        Keyboard.Listen(Key.Down, ButtonState.Down, LiikutaPelaajaaT, "Taakse", pelaaja);
        Keyboard.Listen(Key.Down, ButtonState.Released, Hidasta, null, pelaaja);
        
        Keyboard.Listen(Key.Q, ButtonState.Pressed, Tauko, "Peli pauselle");

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }

/// <summary>
///  Aliohjelma kertoo mitä tapahtuu kun pelaaja kerää kolikon
/// </summary>
/// <param name="pelaaja">kuka kerää</param>
/// <param name="kolikko">minkä kerää</param>
/// Lopputulema: Pelaaja saa pisteen, kolikko katoaa ja jonnekkin ilmaantuu uusi kolikko
///

    public void KeraaKolikon(PhysicsObject Auto, PhysicsObject kolikko)
    {
        pistelaskuri.Value += 1;
        kolikko.Destroy();
        LuoKolikko();
    }

    /// <summary>
    /// Mitä tapahtuu kun pelaaja ja vihollinen törmäävät
    /// </summary>
    /// <param name="Auto"></param>
    /// <param name="vihu"></param>

    public void Kolari(PhysicsObject Auto, PhysicsObject vihu)
    {
        if (vihu.Tag.ToString() == "vihu") ((Auto)Auto).TerveysLaskuri.Value--;
        elamalaskuri.Value --;
        if (pelaaja.TerveysLaskuri.Value <= 0) PeliPaattyy();
    }
    
    /// <summary>
    /// Mitä tapahtuu kun pelaajalta loppuvat elämät
    /// </summary>

    public void PeliPaattyy()
    {
        MultiSelectWindow loppu = new MultiSelectWindow("Peli päättyi", "Alkuun", "Lopeta");
        Add(loppu);
        
        loppu.AddItemHandler(0, Alkuun);
        loppu.AddItemHandler(1, Exit);

    }
    
     /// <summary>
     /// Pelaajan liikkumiseen liittyvät komennot
     /// </summary>
     /// <param name="pelaaja">Olio jota komennot koskevat</param>
     
    public void LiikutaPelaajaaE(PhysicsObject pelaaja)
    {
        Vector pelaajanSuunta = Vector.FromLengthAndAngle(1000.0, pelaaja.Angle);
        pelaaja.Push(pelaajanSuunta);
    }

    public void LiikutaPelaajaaT(PhysicsObject pelaaja)
    {
        Vector pelaajanSuunta = Vector.FromLengthAndAngle(-1000.0, pelaaja.Angle);
        pelaaja.Push(pelaajanSuunta);
    }

    public void KaannaPelaajaav(PhysicsObject auto)
    {
        pelaaja.ApplyTorque(1000);
    }

    public void KaannaPelaajaao(PhysicsObject pelaaja)
    {
        pelaaja.ApplyTorque(-1000);
    }

    public void Pysayta(PhysicsObject pelaaja)
    {
        pelaaja.StopAngular();
    }

    public void Hidasta(PhysicsObject pelaaja)
    {
        pelaaja.LinearDamping = 2.50;
    }

    /// <summary>
    /// Tason luomisen tapahtumat
    /// </summary>

    public void TasonLuominen()
    {
        LuoPelaaja();  //Pelaaja ilmaantuu tasoon
        
        LuoAikalaskuri();    //Pelaajalla on aikalaskuri
        
        LuoVihollinen();   //Luodaan ensimmäinen vihollinen

        LuoPisteLaskuri();  //Pelaajalla on pistelaskuri
        
        LuoElamalaskuri();   //Lisätään palkki näyttämään paljon pelaajaalla on elämää jäljellä
        
        Timer ajastin = new Timer();  //Ajastin tuomaan tasoon uuden vihollisen tietyin väliajoin
        ajastin.Interval = 10;       //Kuinka usein uusia vihollisia ilmaantuu
        ajastin.Timeout += LuoVihollinen;
        ajastin.Start();

        for (int i = 0; i < 3; i++)
        {
            LuoKolikko();
        }                               //Tasoon lisätään 3 kpl kolikoita

        
    }
    
    /// <summary>
    /// Pelin alussa ilmaantuva valikko
    /// </summary>
    
    public void Alkuvalikko()
    {
        MultiSelectWindow alkuvalikko =
            new MultiSelectWindow("Pelin alkuvalikko", "Aloita peli", "Ennätykset", "Lopeta");
        alkuvalikko.AddItemHandler(0, AloitaPeli);      //Siirtyy taso valikkoon
        alkuvalikko.AddItemHandler(1, ParhaatPisteet);  //Näyttää top listat
        alkuvalikko.AddItemHandler(2, Exit);           //Peli suljetaan
        Add(alkuvalikko);
    }

    /// <summary>
    /// Valikko josta valitaan taso
    /// </summary>
    
    public void AloitaPeli()
    {
        MultiSelectWindow kentat =
            new MultiSelectWindow("Valitse taso", "Taso 1", "Taso 2", "Taso 3", "Yllätä mut", "Palaa");
        kentat.AddItemHandler(0, LuoTaso1);   //Luo tason 1
        kentat.AddItemHandler(1, LuoTaso2);   //Luo tason 2
        kentat.AddItemHandler(2, LuoTaso3);   //Luo tason 3
        kentat.AddItemHandler(3, YllataMut);  //Arpoo tasoista 1-3 jonkun
        kentat.AddItemHandler(4, Alkuvalikko);  //Palaa alkuvalikkoon
        
        Add(kentat);
    }

    /// <summary>
    /// Top listat
    /// </summary>
    
    public void ParhaatPisteet()
    {
        Label otsikko = new Label("Parhaat tulokset");
        otsikko.Y = Screen.Top - 50;
        otsikko.TextColor = Color.Black;
        Add(otsikko);
        
        Label teksti1 = new Label("Eniten kolikoita (lkm, pvm, nimi)");
        teksti1.X = Screen.Left + 150;
        teksti1.Y = Screen.Top - 150;
        teksti1.TextColor = Color.Black;
        Add(teksti1);
        
        Label teksti2 = new Label("Pisimpään selvinneet (aika, pvm, nimi)");
        teksti2.X = Screen.Right - 150;
        teksti2.Y = Screen.Top - 150;
        teksti2.TextColor = Color.Black;
        Add(teksti2);

        string[,] kolikoita = {{"lkm", "pvm", "nimi"},{"lkm", "pvm", "nimi"}, {"lkm", "pvm", "nimi"}};
        
    }

    /// <summary>
    /// Tason luovat aliohjelmat
    /// </summary>
    
    public void LuoTaso1()
    {
        TileMap ruudut = TileMap.FromLevelAsset("EkaTaso.txt");
        ruudut.SetTileMethod('#', LuoSeina);
        ruudut.Execute(20, 20);
        Level.CreateBorders();
        
        TasonLuominen();

    }

    public void LuoTaso2()
    {
        TileMap ruudut = TileMap.FromLevelAsset("TokaTaso.txt");
        ruudut.SetTileMethod('#', LuoSeina);
        ruudut.Execute(20, 20);
        Level.CreateBorders();
        
        TasonLuominen();


    }

    public void LuoTaso3()
    {
        TileMap ruudut = TileMap.FromLevelAsset("KolmasTaso.txt");
        ruudut.SetTileMethod('#', LuoSeina);
        ruudut.Execute(20, 20);
        Level.CreateBorders();
        
        TasonLuominen();


    }

    /// <summary>
    /// Aliohjelma, joka arpoo tason
    /// </summary>
    
    public void YllataMut()
    {
        int taso = RandomGen.NextInt(0, 3);
        switch (taso)
        {
            case 0:
                LuoTaso1();
                break;
            case 1:
                LuoTaso2();
                break;
            case 2:
                LuoTaso3();
                break;
        }
    }
    
    /// <summary>
    /// Tasoissa olevien seinien luonti
    /// </summary>
    /// <param name="paikka">Sijainti</param>
    /// <param name="leveys">Leveys mitta</param>
    /// <param name="korkeus">pituus mitta</param>

    public void LuoSeina(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject seina = PhysicsObject.CreateStaticObject(leveys, korkeus);
        seina.Position = paikka;
        seina.Shape = Shape.Rectangle;
        seina.Color = Color.DarkGray;
        seina.Tag = "seina";
        Add(seina);
    }



    /// <summary>
    /// Vihollisen luominen
    /// </summary>
    /// <param name="sijainti"></param>

    public void LuoVihollinen()
    {
        Vihollinen vihu = new Vihollinen(40, 20);
        vihu.Tag = "vihu";
        vihu.Shape = Shape.Rectangle;
        vihu.Color = Color.Red;
        vihu.Position = Level.GetRandomPosition();
        vihu.TerveysLaskuri.Value--;
        AddCollisionHandler(vihu, "seinä", VihuSeinaa);

        FollowerBrain seuraajanAivot = new FollowerBrain(pelaaja);
        vihu.Brain = seuraajanAivot;
        seuraajanAivot.Speed = 200;
        seuraajanAivot.TurnWhileMoving = true;

        Add(vihu);
    }
    
    /// <summary>
    /// Aliohjelma pyrkii saamaan vihollisen pois seinän sisältä
    /// </summary>
    /// <param name="vihu">kuka pois</param>
    /// <param name="seina">mistä pois</param>

    public void VihuSeinaa(PhysicsObject vihu, PhysicsObject seina)
    {
       if (vihu.Tag.ToString() == "vihu") ((Vihollinen)vihu).TerveysLaskuri.Value--;
       
    }
    
     /// <summary>
     /// Luo aikalaskurin
     /// </summary>
     
    public void LuoAikalaskuri()
    {
        Timer aikalaskuri = new Timer();
        aikalaskuri.Start();   //Laskuri alkaa kun taso aloitetaan

        Label aikanaytto = new Label();
        aikanaytto.TextColor = Color.Yellow;
        aikanaytto.DecimalPlaces = 1;
        aikanaytto.BindTo(aikalaskuri.SecondCounter);
        aikanaytto.X = Screen.Right - 40;
        aikanaytto.Y = Screen.Top - 20;
        Add(aikanaytto);
    }
     
     /// <summary>
     /// Luo laskurin, joka laskee pisteitä
     /// </summary>

    public void LuoPisteLaskuri()
    {
        pistelaskuri = new IntMeter(0);    //Laskuri alkaa nollasta

        Label pistenaytto = new Label();
        pistenaytto.X = Screen.Left + 60;
        pistenaytto.Y = Screen.Top - 20;
        pistenaytto.TextColor = Color.Yellow;
        pistenaytto.Title = "Kolikoita: ";

        pistenaytto.BindTo(pistelaskuri);
        Add(pistenaytto);
    }

     /// <summary>
     /// Luo palkin, joka kuvaa paljon pelaajalla on elämää jäljellä
     /// </summary>
     
     public void LuoElamalaskuri()
     {
         elamalaskuri = new DoubleMeter(4);
         elamalaskuri.MaxValue = 4;
         

         ProgressBar elamapalkki = new ProgressBar(150, 20);
         elamapalkki.Y = Screen.Top - 20;
         elamapalkki.BindTo(elamalaskuri);
         Add(elamapalkki);
     }
     
    /// <summary>
    /// Kolikon luonti
    /// </summary>

    public void LuoKolikko()
    {
        PhysicsObject kolikko = new PhysicsObject(10, 10);  //Kolikon mitat
        kolikko.Tag = "kolikko";                                       //Tagi johon kolikko on liitetty
        Vector sijainti = Level.GetRandomPosition();                //Kolikon sijainti tasossa on satunnainen
        kolikko.Position = sijainti;
        kolikko.Shape = Shape.Circle;
        kolikko.Color = Color.Yellow;
        Add(kolikko);
        AddCollisionHandler(kolikko, "seina", Jumi);        //Estää kolikon jäämistä jumiin seinän sisään
    }

    /// <summary>
    /// Estää kolikon jäämistä seinän sisään
    /// </summary>

    
    public void Jumi(PhysicsObject kolikko, PhysicsObject seina)
    {
        kolikko.Destroy();
        LuoKolikko();
    }
    
    /// <summary>
    /// Luo taukovalikon
    /// </summary>
    
    private void Tauko()
   {
       if (IsPaused)
       {
           Pausevalikko();
       }
       else
       { 
            Pausevalikko();
       }

       
       Pause();
    }
    
    /// <summary>
    /// Taukovalikon sisältö
    /// </summary>
    
    public void Pausevalikko()
    {
        MultiSelectWindow tauko = new MultiSelectWindow("Pause", "Jatka", "Alkuvalikkoon");
        tauko.AddItemHandler(0, Pause);         //Jatka painike jatkaa peliä
        tauko.AddItemHandler(1, Alkuun);       //Alkuun painike vie pelaajan alkuvalikkoon
        
        Add(tauko);
        
    }
    
    /// <summary>
    /// Aliohjelma suorittaa tarvittavat toiminnot alkuvalikkoon pääsemisesksi
    /// </summary>
    
    public void Alkuun()
    {
        ClearAll();
        Alkuvalikko();
    }

}

